@@ -237,7 +237,12 @@ void uwsgi_detach_daemons() {
 			time_t timeout = uwsgi_now() + (uwsgi.reload_mercy ? uwsgi.reload_mercy : 3);
 			int waitpid_status;
 			while (!kill(ud->pid, 0)) {
-				kill(-(ud->pid), ud->stop_signal);
+				if (uwsgi_instance_is_reloading && ud->reload_signal > 0) {
+					kill(-(ud->pid), ud->reload_signal);
+				}
+				else {
+					kill(-(ud->pid), ud->stop_signal);
+				}
 				sleep(1);
 				waitpid(ud->pid, &waitpid_status, WNOHANG);
 				if (uwsgi_now() >= timeout) {
