@@ -190,9 +190,11 @@ safe:
 		time_t oldest_worker_spawn = INT_MAX;
 		for (i = 1; i <= uwsgi.numproc; i++) {
 			if (uwsgi.workers[i].cheaped == 0 && uwsgi.workers[i].pid > 0) {
-				if (uwsgi.workers[i].last_spawn < oldest_worker_spawn) {
-					oldest_worker_spawn = uwsgi.workers[i].last_spawn;
-					oldest_worker = i;
+				if (uwsgi_worker_is_busy(i) == 0) {
+					if (uwsgi.workers[i].last_spawn < oldest_worker_spawn) {
+						oldest_worker_spawn = uwsgi.workers[i].last_spawn;
+						oldest_worker = i;
+					}
 				}
 			}
 		}
@@ -390,6 +392,8 @@ void uwsgi_reload(char **argv) {
 	int i;
 	int waitpid_status;
 
+	if (uwsgi.new_argv) argv = uwsgi.new_argv;
+
 	if (!uwsgi.master_is_reforked) {
 
 		// call a series of waitpid to ensure all processes (gateways, mules and daemons) are dead
@@ -1198,6 +1202,18 @@ struct uwsgi_stats *uwsgi_master_generate_stats() {
 			if (uwsgi_stats_list_close(us))
 				goto end;
 
+			if (uwsgi_stats_comma(us)) goto end;
+
+			if (uwsgi_stats_key(us, "req_info"))
+				goto end;
+
+			if (uwsgi_stats_object_open(us))
+				goto end;
+
+			if (uwsgi_stats_dump_request(us, uc)) goto end;
+
+			if (uwsgi_stats_object_close(us))
+				goto end;
 
 			if (uwsgi_stats_object_close(us))
 				goto end;
