@@ -14,7 +14,9 @@ static void uwsgi_plugin_parse_section(char *filename) {
 			if (equal) {
 				*equal = 0;
 				if (!strcmp(p, "requires")) {
-					uwsgi_load_plugin(-1, equal + 1, NULL);
+					if (!plugin_already_loaded(equal+1)) {	
+						uwsgi_load_plugin(-1, equal + 1, NULL);
+					}
 				}
 			}
 			p = strtok_r(NULL, "\n", &ctx);
@@ -84,7 +86,7 @@ void *uwsgi_load_plugin(int modifier, char *plugin, char *has_option) {
 
 	int need_free = 0;
 	char *plugin_name = uwsgi_strip(uwsgi_str(plugin));
-	char *plugin_symbol_name_start = plugin_name;
+	char *plugin_symbol_name_start;
 
 	struct uwsgi_plugin *up;
 	char linkpath_buf[1024], linkpath[1024];
