@@ -4,6 +4,13 @@ then
   echo "Usage: install.sh <profile> <binary_path>"
   exit 1
 fi
+
+if [ ${2:0:1} != "/" ]
+then
+  echo "uWSGI binary path must be absolute !!!"
+  exit 1
+fi
+
 echo "downloading latest uWSGI tarball..."
 curl -o uwsgi_latest_from_installer.tar.gz http://projects.unbit.it/downloads/uwsgi-latest.tar.gz
 mkdir uwsgi_latest_from_installer
