@@ -374,6 +374,12 @@ realstuff:
 			PyErr_Clear();
 	}
 
+	// For the reasons explained in
+	// https://github.com/unbit/uwsgi/issues/1384, tearing down
+	// the interpreter can be very expensive.
+	if (uwsgi.skip_atexit_teardown)
+		return;
+
 	Py_Finalize();
 }
 
@@ -524,16 +530,24 @@ void init_uwsgi_vars() {
 	pysys_dict = PyModule_GetDict(pysys);
 
 #ifdef PYTHREE
-	// fix stdout and stderr
+	// In python3, stdout / stderr was changed to be buffered (a bug according
+	// to many):
+	// - https://bugs.python.org/issue13597
+	// - https://bugs.python.org/issue13601
+	// We'll fix this by manually restore the unbuffered behaviour.
+	// In the case of a tty, this fix breaks readline support in interactive
+	// debuggers so we'll only do this in the non-tty case.
+	if (!Py_FdIsInteractive(stdin, NULL)) {
 #ifdef HAS_NO_ERRORS_IN_PyFile_FromFd
-	PyObject *new_stdprint = PyFile_FromFd(2, NULL, "w", _IOLBF, NULL, NULL, 0);
+		PyObject *new_stdprint = PyFile_FromFd(2, NULL, "w", _IOLBF, NULL, NULL, 0);
 #else
-	PyObject *new_stdprint = PyFile_FromFd(2, NULL, "w", _IOLBF, NULL, NULL, NULL, 0);
+		PyObject *new_stdprint = PyFile_FromFd(2, NULL, "w", _IOLBF, NULL, NULL, NULL, 0);
 #endif
-	PyDict_SetItemString(pysys_dict, "stdout", new_stdprint);
-	PyDict_SetItemString(pysys_dict, "__stdout__", new_stdprint);
-	PyDict_SetItemString(pysys_dict, "stderr", new_stdprint);
-	PyDict_SetItemString(pysys_dict, "__stderr__", new_stdprint);
+		PyDict_SetItemString(pysys_dict, "stdout", new_stdprint);
+		PyDict_SetItemString(pysys_dict, "__stdout__", new_stdprint);
+		PyDict_SetItemString(pysys_dict, "stderr", new_stdprint);
+		PyDict_SetItemString(pysys_dict, "__stderr__", new_stdprint);
+	}
 #endif
 	pypath = PyDict_GetItemString(pysys_dict, "path");
 	if (!pypath) {
@@ -1043,17 +1057,16 @@ void *uwsgi_python_create_env_holy(struct wsgi_request *wsgi_req, struct uwsgi_a
 	Py_INCREF(up.wsgi_spitout);
 	PyTuple_SetItem((PyObject *)wsgi_req->async_args, 1, up.wsgi_spitout);
 	PyObject *env = PyDict_New();
-	Py_INCREF(env);
 	return env;
 }
 
 void uwsgi_python_destroy_env_holy(struct wsgi_request *wsgi_req) {
-	Py_DECREF((PyObject *)wsgi_req->async_environ);
-	Py_DECREF((PyObject *) wsgi_req->async_args);
 	// in non-multithread modes, we set uwsgi.env incrementing the refcount of the environ
 	if (uwsgi.threads < 2) {
 		Py_DECREF((PyObject *)wsgi_req->async_environ);
 	}
+	Py_DECREF((PyObject *) wsgi_req->async_args);
+	Py_DECREF((PyObject *)wsgi_req->async_environ);
 }
 
 
@@ -1814,7 +1827,20 @@ int uwsgi_python_mule(char *opt) {
 		UWSGI_RELEASE_GIL;
 		return 1;
 	}
-	
+	else if (strchr(opt, ':')) {
+		UWSGI_GET_GIL;
+		PyObject *result = NULL;
+		PyObject *arglist = Py_BuildValue("()");
+		PyObject *callable = up.loaders[LOADER_MOUNT](opt);
+		if (callable) {
+			result = PyEval_CallObject(callable, arglist);
+		}
+		Py_XDECREF(result);
+		Py_XDECREF(arglist);
+		Py_XDECREF(callable);
+		UWSGI_RELEASE_GIL;
+		return 1;
+	}
 	return 0;
 	
 }
@@ -1895,7 +1921,7 @@ static ssize_t uwsgi_python_logger(struct uwsgi_logger *ul, char *message, size_
 		PyObject *py_getLogger_args = NULL;
 		if (ul->arg) {
 			py_getLogger_args = PyTuple_New(1);
-			PyTuple_SetItem(py_getLogger_args, 0, PyString_FromString(ul->arg));
+			PyTuple_SetItem(py_getLogger_args, 0, UWSGI_PYFROMSTRING(ul->arg));
 		}
                 ul->data = (void *) PyEval_CallObject(py_getLogger, py_getLogger_args);
                 if (PyErr_Occurred()) {
