@@ -83,7 +83,8 @@ retry:
         }
 
         else if (err == SSL_ERROR_SYSCALL) {
-                uwsgi_error("uwsgi_proto_suwsgi_parser()/SSL_read()");
+		if (errno != 0)
+                	uwsgi_error("uwsgi_proto_suwsgi_parser()/SSL_read()");
         }
 
         return -1;
@@ -219,6 +220,7 @@ void uwsgi_proto_uwsgi_setup(struct uwsgi_socket *uwsgi_sock) {
 	uwsgi_sock->proto_fix_headers = uwsgi_proto_base_fix_headers;
 	uwsgi_sock->proto_read_body = uwsgi_proto_base_read_body;
 	uwsgi_sock->proto_write = uwsgi_proto_base_write;
+	uwsgi_sock->proto_writev = uwsgi_proto_base_writev;
 	uwsgi_sock->proto_write_headers = uwsgi_proto_base_write;
 	uwsgi_sock->proto_sendfile = uwsgi_proto_base_sendfile;
 	uwsgi_sock->proto_close = uwsgi_proto_base_close;
