@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '2.0.12'
+uwsgi_version = '2.0.15'
 
 import os
 import re
@@ -27,9 +27,15 @@ try:
 except:
     import configparser as ConfigParser
 
-GCC = os.environ.get('CC', sysconfig.get_config_var('CC'))
-if not GCC:
-    GCC = 'gcc'
+
+PY3 = sys.version_info[0] == 3
+
+if uwsgi_os == 'Darwin':
+    GCC = os.environ.get('CC', 'clang')
+else:
+    GCC = os.environ.get('CC', sysconfig.get_config_var('CC'))
+    if not GCC:
+        GCC = 'gcc'
 
 def get_preprocessor():
     if 'clang' in GCC:
@@ -82,6 +88,7 @@ report = {
     'debug': False,
     'plugin_dir': False,
     'zlib': False,
+    'ucontext': False,
 }
 
 verbose_build = False
@@ -158,6 +165,42 @@ def spcall2(cmd):
     else:
         return None
 
+
+def test_snippet(snippet):
+    """Compile a C snippet to see if features are available at build / link time."""
+    if sys.version_info[0] >= 3 or (sys.version_info[0] == 2 and sys.version_info[1] > 5):
+        if not isinstance(snippet, bytes):
+            if PY3:
+                snippet = bytes(snippet, sys.getdefaultencoding())
+            else:
+                snippet = bytes(snippet)
+        cmd = "{0} -xc - -o /dev/null".format(GCC)
+    else:
+        cmd = GCC + " -xc - -o /dev/null"
+    p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
+    p.communicate(snippet)
+    return p.returncode == 0
+
+
+def has_usable_ucontext():
+    if uwsgi_os in ('OpenBSD', 'Haiku'):
+        return False
+    if uwsgi_os.startswith('CYGWIN'):
+        return False
+    if uwsgi_os == 'Darwin' and uwsgi_os_k.startswith('8'):
+        return False
+    if uwsgi_cpu[0:3] == 'arm':
+        return False
+    # check for ucontext.h functions definitions, musl has only declarations
+    return test_snippet("""#include <ucontext.h>
+int main()
+{
+	ucontext_t uc;
+	getcontext(&uc);
+	return 0;
+}""")
+
+
 def spcall3(cmd):
     p = subprocess.Popen(cmd, shell=True, stdin=open('/dev/null'), stderr=subprocess.PIPE, stdout=subprocess.PIPE)
 
@@ -257,7 +300,7 @@ def build_uwsgi(uc, print_only=False, gcll=None):
             if not p or p == 'None':
                 continue
             if p == 'ugreen':
-                if uwsgi_os == 'OpenBSD' or uwsgi_cpu[0:3] == 'arm' or uwsgi_os == 'Haiku' or uwsgi_os.startswith('CYGWIN') or (uwsgi_os == 'Darwin' and uwsgi_os_k.startswith('8')):
+                if not report['ucontext']:
                     continue
             epc += "UDEP(%s);" % p
             eplc += "ULEP(%s);" % p
@@ -299,7 +342,6 @@ def build_uwsgi(uc, print_only=False, gcll=None):
     ulc.write(uwsgi_cflags)
     ulc.close()
 
-
     # embed uwsgi.h in the server binary. It increases the binary size, but will be very useful
     # for various tricks (like cffi integration)
     # if possibile, the blob is compressed
@@ -398,7 +440,7 @@ def build_uwsgi(uc, print_only=False, gcll=None):
                     continue
 
                 if p == 'ugreen':
-                    if uwsgi_os == 'OpenBSD' or uwsgi_cpu[0:3] == 'arm' or uwsgi_os == 'Haiku' or uwsgi_os.startswith('CYGWIN') or (uwsgi_os == 'Darwin' and uwsgi_os_k.startswith('8')):
+                    if not report['ucontext']:
                         continue
 
                 path = path.rstrip('/')
@@ -604,7 +646,7 @@ class uConf(object):
             if last_profile != filename:
                 os.environ['UWSGI_FORCE_REBUILD'] = '1'
 
-        ulp = open('uwsgibuild.lastprofile','w')
+        ulp = open('uwsgibuild.lastprofile', 'w')
         ulp.write(filename)
         ulp.close()
 
@@ -630,7 +672,7 @@ class uConf(object):
             self.include_path += os.environ['UWSGI_INCLUDES'].split(',')
 
 
-        self.cflags = ['-O2', '-I.', '-Wall', '-Werror', '-D_LARGEFILE_SOURCE', '-D_FILE_OFFSET_BITS=64'] + os.environ.get("CFLAGS", "").split() + self.get('cflags','').split()
+        self.cflags = ['-O2', '-I.', '-Wall', '-D_LARGEFILE_SOURCE', '-D_FILE_OFFSET_BITS=64'] + os.environ.get("CFLAGS", "").split() + self.get('cflags','').split()
 
         report['kernel'] = uwsgi_os
 
@@ -806,9 +848,10 @@ class uConf(object):
 
         if uwsgi_os == 'OpenBSD':
             try:
-                obsd_major = int(uwsgi_os_k.split('.')[0])
-                obsd_minor = int(uwsgi_os_k.split('.')[1])
-                if obsd_major >= 5 and obsd_minor > 0:
+                obsd_major = uwsgi_os_k.split('.')[0]
+                obsd_minor = uwsgi_os_k.split('.')[1]
+                obsd_ver = int(obsd_major + obsd_minor)
+                if obsd_ver > 50:
                     self.cflags.append('-DUWSGI_NEW_OPENBSD')
                     report['kernel'] = 'New OpenBSD'
             except:
@@ -850,6 +893,9 @@ class uConf(object):
             for extra in extras.split(','):
                 self.gcc_list.append(extra)
 
+        # check for usable ucontext
+        report['ucontext'] = has_usable_ucontext()
+
         # set locking subsystem
         locking_mode = self.get('locking','auto')
 
@@ -1378,7 +1424,7 @@ def build_plugin(path, uc, cflags, ldflags, libs, name = None):
         pass
 
     if uc:
-        plugin_dest = uc.get('plugin_dir') + '/' + name + '_plugin'
+        plugin_dest = uc.get('plugin_build_dir', uc.get('plugin_dir')) + '/' + name + '_plugin'
     else:
         plugin_dest = name + '_plugin'
 
